@@ -19,6 +19,7 @@
  */
 
 const Lang = imports.lang;
+const Signals = imports.signals;
 
 const Shell = imports.gi.Shell;
 
@@ -31,71 +32,58 @@ const Extension = imports.misc.extensionUtils.getCurrentExtension();
 const VIDEO_PLAYER_CATEGORIES = [
     ['Player', 'Video'],
     ['Player', 'AudioVideo'],
-    ['Game'],
+    ['Game']
 ];
 
 
-const Hook = new Lang.Class({
-    Name: 'PomodoroHook',
-
-    _init: function(object, property, func) {
-        this.object = object;
-        this.property = property;
-        this.func = func;
-
-        this.initial = object[property];
-
-        this.check();
-    },
-
-    check: function() {
-        if (this.initial === undefined) {
-            logWarning('Hook "%s" for %s is not defined'.format(this.property, this.object));
-            return;
-        }
-        if (!(this.initial instanceof Function)) {
-            logWarning('Hook "%s" for %s is not callable'.format(this.property, this.object));
-            return;
-        }
-    },
-
-    override: function(func) {
-        this.object[this.property] = func ? func : this.func;
-    },
-
-    restore: function() {
-        this.object[this.property] = this.initial;
-    }
-});
-
-
 const Patch = new Lang.Class({
     Name: 'PomodoroPatch',
 
-    _init: function() {
-        this._hooks = [];
-    },
+    _init: function(object, overrides) {
+        this.object = object;
+        this.overrides = overrides;
+        this.initial = {};
+        this.applied = false;
 
-    addHooks: function(object, hooks) {
-        for (let name in hooks) {
-            this._hooks.push(new Hook(object, name, hooks[name]));
-        };
+        for (let name in this.overrides) {
+            this.initial[name] = this.object[name];
+
+            if (typeof(this.initial[name]) == 'undefined') {
+                logWarning('Property "%s" for %s is not defined'.format(name, this.object));
+            }
+        }
     },
 
     apply: function() {
-        this._hooks.forEach(Lang.bind(this,
-            function(hook) {
-                hook.override();
-            }));
+        if (!this.applied) {
+            for (let name in this.overrides) {
+                this.object[name] = this.overrides[name];
+            }
+
+            this.applied = true;
+
+            this.emit('applied');
+        }
     },
 
     revert: function() {
-        this._hooks.forEach(Lang.bind(this,
-            function(hook) {
-                hook.restore();
-            }));
+        if (this.applied) {
+            for (let name in this.overrides) {
+                this.object[name] = this.initial[name];
+            }
+
+            this.applied = false;
+
+            this.emit('reverted');
+        }
+    },
+
+    destroy: function() {
+        this.revert();
+        this.disconnectAll();
     }
 });
+Signals.addSignalMethods(Patch.prototype);
 
 
 function arrayContains(array1, array2) {
